impl From<ZitadelConfig> for OAuth {
    fn from(value: ZitadelConfig) -> Self {
        Self::new_zitadel(value)
    }
}

use async_trait::async_trait;

use oauth2::{
    basic::BasicClient, reqwest::async_http_client, url::Url, AuthUrl, AuthorizationCode, ClientId,
    ClientSecret, CsrfToken, RedirectUrl, Scope, TokenResponse, TokenUrl,
};

use crate::login::auth_clap::ZitadelClap;

use super::{OAuth, OAuthClient};

#[derive(Clone)]
pub struct ZitadelConfig {
    auth_url: String,
    client_id: String,
    client_secret: String,
    redirect_url: String,
    token_url: String,
    authority_url: String,
}

pub struct ZitadelOAuthClient {
    client: BasicClient,
}

impl ZitadelOAuthClient {
    pub fn new(
        client_id: impl Into<String>,
        client_secret: impl Into<String>,
        redirect_url: impl Into<String>,
        auth_url: impl Into<String>,
        token_url: impl Into<String>,
        authority_url: impl Into<String>,
    ) -> Self {
        Self {
            client: Self::oauth_client(ZitadelConfig {
                client_id: client_id.into(),
                client_secret: client_secret.into(),
                redirect_url: redirect_url.into(),
                auth_url: auth_url.into(),
                token_url: token_url.into(),
                authority_url: authority_url.into(),
            }),
        }
    }

    fn oauth_client(config: ZitadelConfig) -> BasicClient {
        BasicClient::new(
            ClientId::new(config.client_id),
            Some(ClientSecret::new(config.client_secret)),
            AuthUrl::new(config.auth_url).unwrap(),
            Some(TokenUrl::new(config.token_url).unwrap()),
        )
        .set_redirect_uri(RedirectUrl::new(config.redirect_url).unwrap())
    }
}

impl From<ZitadelConfig> for ZitadelOAuthClient {
    fn from(value: ZitadelConfig) -> Self {
        Self::new(
            value.client_id,
            value.client_secret,
            value.redirect_url,
            value.auth_url,
            value.token_url,
            value.authority_url,
        )
    }
}

impl TryFrom<ZitadelClap> for ZitadelConfig {
    type Error = anyhow::Error;

    fn try_from(value: ZitadelClap) -> Result<Self, Self::Error> {
        let authority_url = value
            .authority_url
            .ok_or(anyhow::anyhow!("authority_url was not set"))?;

        Ok(Self {
            auth_url: format!("{}/oauth/v2/authorize", &authority_url),
            token_url: format!("{}/oauth/v2/token", &authority_url),
            authority_url,
            client_id: value
                .client_id
                .ok_or(anyhow::anyhow!("client_id was not set"))?,
            client_secret: value
                .client_secret
                .ok_or(anyhow::anyhow!("client_secret was not set"))?,
            redirect_url: value
                .redirect_url
                .ok_or(anyhow::anyhow!("redirect_url was not set"))?,
        })
    }
}

#[async_trait]
impl OAuthClient for ZitadelOAuthClient {
    async fn get_token(&self) -> anyhow::Result<()> {
        Ok(())
    }
    async fn authorize_url(&self) -> anyhow::Result<Url> {
        let (auth_url, _csrf_token) = self
            .client
            .authorize_url(CsrfToken::new_random)
            .add_scope(Scope::new("identify".to_string()))
            .add_scope(Scope::new("openid".to_string()))
            .add_scope(Scope::new("email".to_string()))
            .add_scope(Scope::new("profile".to_string()))
            .url();

        Ok(auth_url)
    }

    async fn exchange(&self, code: &str) -> anyhow::Result<String> {
        let token = self
            .client
            .exchange_code(AuthorizationCode::new(code.to_string()))
            .request_async(async_http_client)
            .await?;

        Ok(token.access_token().secret().clone())
    }
}
